# -*- coding:Utf-8 -*-
# ../addons/eventscripts/_libs/python/insertcmdlib.py
# allow you to execute a server command. Can execute a function when command has been passed

import es, psyco
psyco.full()



esInfo = es.AddonInfo()
esInfo.author = "DoCky"
esInfo.version = "1.101p"
esInfo.name = "InsertCmdLib"
esInfo.basename = "insertcmd"
esInfo.url = "http://addons.eventscripts.com/addons/view/insertcmdlib"
esInfo.description = "allow you to execute a server command. Can execute a ESP function when command has been passed"

version = es.ServerVar("insertcmdlib", "", " Created by DoCky -")
version.set(esInfo.version)
version.makepublic()



# use es.server.insertcmd for inject command in server's console
_insertcmd = es.server.insertcmd

class InsertQueue(list):
    """
        InsertQueue object
            List with 2 functions for manage inserted command
    """

    def _insertcmd(self):
        try:
            # get instance of current running command
            instance = self[0]
            # remove command from queue
            self.remove(instance)
            # set status of execution
            instance.executed = True
            # execute callback function with InsertCmd object as parameter
            if callable(instance.function): instance.function(instance)
            # if commands remains in list, execute them...
            if self: self[0]._prompt()
        # debug message...
        except IndexError: es.dbgmsg(1, "insertcmdlib: command has been canceled or script is no longer active")

    def getInsertCmd(self, name):
        # return instance with parameter "name" or None
        for instance in self:
            if instance.name == name: return instance

# store InsertQueue object
_insertqueue = InsertQueue()
# register funbctions for internal command
es.addons.registerBlock("_insertcmdlib", "_insertcmd", _insertqueue._insertcmd)
# register server commands
es.regcmd("_icl-cmd_", "_insertcmdlib/_insertcmd", "insertcmdlib: internal command")



# create an object for each inserted command
class InsertCmd(object):
    """
        InsertCmd object
            Store informations for an inserted command
            Allow you to execute or cancel it
    """

    def __init__(self, command = "", function = None, parameters = (), name = ""):
        # store command
        self.command = str(command) if command else ""
        # store function to call back
        self.function = function if function else None
        # store parameters
        self.parameters = parameters if parameters else ()
        # store name of thread (like a targetname)
        self.name = str(name) if name else ""
        # set True/False if command has been injected
        self.injected = False
        # set status of command (True if it was executed, else False)
        self.executed = False

    def __getitem__(self, name):
        # return attribute if it exists
        return getattr(self, name)

    def _prompt(self):
        # insert command in server's queue
        _insertcmd(self.command + ";_icl-cmd_")
        # set True for injected attribute
        self.injected = True

    def execute(self):
        """ execute command """
        # set status of execution
        self.injected = False
        self.executed = False
        # insert command into the queue
        _insertqueue.append(self)
        # if command is the only, execute it...
        if len(_insertqueue) == 1: self._prompt()

    def cancel(self):
        """ cancel registered command """
        # if command has been inserted in queue
        if self in _insertqueue:
            # remove it and return True
            _insertqueue.remove(self)
            return True
        # else, return False
        return False



# execute command...
def execute(command = "", function = None, parameters = (), name = ""):
    # create and get InsertCmd instance
    insertcmd = InsertCmd(command, function, parameters, name)
    # add command in queue
    insertcmd.execute()
    # return instance
    return insertcmd

# cancel command
def cancel(name):
    # try to find instance
    instance = getInsertCmd(name)
    # if command does not existing, return False
    if instance is None: return False
    # while instances with <name> exists
    while instance:
        # remove it
        _insertqueue.remove(instance)
        instance = getInsertCmd(name)
    # return True
    return True

# get instance
def getInsertCmd(name):
    # get instance by <name> or return None
    return _insertqueue.getInsertCmd(name)
